use std::sync::Mutex;

use actix_web::web;
use diesel::{r2d2, PgConnection};

type DbPool = r2d2::Pool<r2d2::ConnectionManager<PgConnection>>;
type CachePool = r2d2::Pool<redis::Client>;

pub struct Connections {
    pub db: DbPool,
    pub cache: CachePool,
}

pub struct ConnectionsInit {
    pub db_init: DatabaseInit,
    // smtp_init: SmtpInit,
    pub cache_init: CacheInit,
}

pub struct DatabaseInit {
    pub url: String,
}

// struct SmtpInit {
//     server: String,
//     usernam: String,
//     password: String,
// }

pub struct CacheInit {
    pub url: String,
}

pub type ConnectionsHandle = web::Data<Mutex<Connections>>;

//#todo!() have a better Error handling, I stopped searching because of r2d2::Error not implementing some traits.

impl Connections {
    pub fn new(init: ConnectionsInit) -> Result<Self, ()> {
        let db_manager = r2d2::ConnectionManager::<PgConnection>::new(init.db_init.url);
        let db_connection = r2d2::Pool::builder().build(db_manager).unwrap();

        // let smtp_connection = todo!();

        // not sure if this is correctly building a pooled connection
        let cache_manager = redis::Client::open(init.cache_init.url).unwrap();
        let cache_connection = r2d2::Pool::builder().build(cache_manager).unwrap();

        Ok(Connections {
            db: db_connection,
            // smtp: smtp_connection,
            cache: cache_connection,
        })
    }
}

#[cfg(test)]
mod tests {
    use super::CacheInit;
    use super::Connections;
    use super::ConnectionsInit;
    use super::DatabaseInit;
    use std::env;

    #[test]
    fn test_connections_init() {
        dotenvy::dotenv().ok();
        let database_url = env::var("DATABASE_URL").expect("DATABASE_URL must be set");
        let cache_url = env::var("TEST_CACHE_URL").expect("TEST_CACHE_URL must be set");

        let connections = Connections::new(ConnectionsInit {
            db_init: DatabaseInit { url: database_url },
            cache_init: CacheInit { url: cache_url },
        });
        assert!(connections.is_ok());
    }
}
