use crate::model::structures::projects::Project;
use crate::schema::datasets;
use camino::Utf8PathBuf;
use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(
    Selectable,
    Queryable,
    Identifiable,
    Insertable,
    Deserialize,
    Serialize,
    Eq,
    PartialEq,
    Debug,
    Clone,
    Associations,
)]
#[diesel(primary_key(id))]
#[diesel(belongs_to(Project))]
pub struct Dataset {
    pub id: Uuid,
    #[diesel(serialize_as = String, deserialize_as = String)]
    pub path: Utf8PathBuf,
    pub project_id: Option<Uuid>,
}

#[derive(AsChangeset, Deserialize, Clone)]
#[diesel(table_name = datasets)]
pub struct UpdateDataset {
    #[diesel(serialize_as = String, deserialize_as = String)]
    pub path: Option<Utf8PathBuf>,
    pub project_id: Option<Uuid>,
}
