use crate::{model::structures::projects::Project, schema::extension_dates};
use chrono::NaiveDateTime;
use diesel::sql_types::Timestamp;
use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(
    Selectable,
    Queryable,
    Identifiable,
    Associations,
    Insertable,
    Deserialize,
    Serialize,
    Eq,
    PartialEq,
    Debug,
    Clone,
    AsExpression,
)]
#[diesel(primary_key(id))]
#[diesel(belongs_to(Project))]
#[diesel(sql_type = Timestamp)]
#[diesel(table_name = extension_dates)]
pub struct ExtensionDate {
    pub id: Uuid,
    pub extension_end_date: NaiveDateTime,
    pub project_id: Option<Uuid>,
}

#[derive(AsChangeset, Deserialize, Clone)]
#[diesel(table_name = extension_dates)]
pub struct UpdateExtensionDate {
    pub extension_end_date: Option<NaiveDateTime>,
    pub project_id: Option<Uuid>,
}
