use crate::model::structures::users::User;
use crate::schema::projects;
use camino::Utf8PathBuf;
use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(
    Selectable,
    Queryable,
    Identifiable,
    Associations,
    Insertable,
    Deserialize,
    Serialize,
    Eq,
    PartialEq,
    Debug,
    Clone,
)]
#[diesel(table_name = projects)]
#[diesel(primary_key(id))]
#[diesel(belongs_to(User))]
pub struct Project {
    pub id: Uuid,
    pub name: String,
    #[diesel(serialize_as = String, deserialize_as = String)]
    pub path: Utf8PathBuf,
    pub user_id: Option<i32>,
}

#[derive(AsChangeset, Deserialize)]
#[diesel(table_name = projects)]
pub struct UpdateProject {
    pub name: Option<String>,
    #[diesel(serialize_as = String, deserialize_as = String)]
    pub path: Option<Utf8PathBuf>,
    pub user_id: Option<i32>,
}
