use diesel::result::Error as DatabaseError;
use redis::RedisError;
use std::fmt::{self, Display};

#[derive(Debug)]
pub enum Error {
    Model(DatabaseError),
    PackageNotInstalled(String),
    Cache(RedisError),
}

impl Display for Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Error::Model(error) => {
                write!(f, "Error while trying to access the database: {error}")
            }
            Error::PackageNotInstalled(package_name) => write!(
                f,
                "{package_name} is not installed, please install {package_name} and try again"
            ),
            Error::Cache(error) => write!(f, "Failed to communicate with redis: {error}"),
        }
    }
}
