use actix_web::{delete, error, get, post, put, web, HttpResponse, Responder};
use uuid::Uuid;

use crate::{
    connections::ConnectionsHandle,
    model::structures::extension_dates::{ExtensionDate, UpdateExtensionDate},
};

#[post("/")]
async fn insert_with_id(
    _: crate::router::auth::JWTClaims,
    connections: ConnectionsHandle,
    t: web::Json<ExtensionDate>,
) -> actix_web::Result<impl Responder> {
    let extension_date = web::block(move || {
        let connections = connections.lock().unwrap();
        let mut connection = connections
            .db
            .get()
            .expect("Couldn't get db connection from pool");
        crate::model::operations::extension_dates::insert_with_id(&mut connection, t.into_inner())
    })
    .await?
    .map_err(error::ErrorInternalServerError)?;
    Ok(HttpResponse::Ok().json(extension_date))
}

#[get("/{extension_date_id}")]
async fn find_by_id(
    connections: ConnectionsHandle,
    extension_date_id: web::Path<Uuid>,
) -> actix_web::Result<impl Responder> {
    let id = extension_date_id.into_inner();
    let extension_date = web::block(move || {
        let connections = connections.lock().unwrap();
        let mut connection = connections
            .db
            .get()
            .expect("Couldn't get db connection from pool");
        crate::model::operations::extension_dates::find_by_id(&mut connection, id)
    })
    .await?
    .map_err(error::ErrorInternalServerError)?;
    match extension_date {
        Some(extension_date) => Ok(HttpResponse::Ok().json(extension_date)),
        None => Ok(HttpResponse::NotFound().into()),
    }
}

#[get("/projects/{project_id}")]
async fn find_by_project(
    connections: ConnectionsHandle,
    project_id: web::Path<Uuid>,
) -> actix_web::Result<impl Responder> {
    let p_id = project_id.into_inner();
    let request = web::block(move || {
        let connections = connections.lock().unwrap();
        let mut connection = connections
            .db
            .get()
            .expect("Couldn't get db connection from pool");
        crate::model::operations::extension_dates::find_by_project(&mut connection, p_id)
    })
    .await?
    .map_err(error::ErrorInternalServerError)?;
    Ok(HttpResponse::Ok().json(request))
}

#[put("/{extension_date_id}")]
async fn update(
    connections: ConnectionsHandle,
    user_id: web::Path<Uuid>,
    updated_extension_date_info: web::Json<UpdateExtensionDate>,
) -> actix_web::Result<impl Responder> {
    let extension_date_updated = web::block(move || {
        let connections = connections.lock().unwrap();
        let mut connection = connections
            .db
            .get()
            .expect("Coulnd't get db connection from pool");
        crate::model::operations::extension_dates::update(
            &mut connection,
            user_id.into_inner(),
            updated_extension_date_info.into_inner(),
        )
    })
    .await?
    .map_err(error::ErrorInternalServerError);
    match extension_date_updated {
        Ok(extension_dates_updated_count) => {
            Ok(HttpResponse::Ok().json(extension_dates_updated_count))
        }
        Err(error) => Err(error),
    }
}

#[delete("/{extension_date_id}")]
async fn delete(
    connections: ConnectionsHandle,
    extension_date_id: web::Path<Uuid>,
) -> actix_web::Result<impl Responder> {
    let id = extension_date_id.into_inner();
    let deleted_extension_date_count = web::block(move || {
        let connections = connections.lock().unwrap();
        let mut connection = connections
            .db
            .get()
            .expect("Couldn't get db connection from pool");
        crate::model::operations::extension_dates::delete(&mut connection, id)
    })
    .await?
    .map_err(error::ErrorInternalServerError)?;
    match deleted_extension_date_count {
        0 => Ok(HttpResponse::NotFound()),
        _ => Ok(HttpResponse::Ok()),
    }
}
