use actix_web::{delete, error, get, post, web, HttpResponse, Responder};

use crate::{connections::ConnectionsHandle, model::structures::teams::Team};

#[post("/")]
async fn insert_with_id(
    _: crate::router::auth::JWTClaims,
    connections: ConnectionsHandle,
    t: web::Json<Team>,
) -> actix_web::Result<impl Responder> {
    let team = web::block(move || {
        let connections = connections.lock().unwrap();
        let mut connection = connections
            .db
            .get()
            .expect("Couldn't get db connection from pool");
        crate::model::operations::teams::insert_with_id(&mut connection, t.into_inner())
    })
    .await?
    .map_err(error::ErrorInternalServerError)?;
    Ok(HttpResponse::Ok().json(team))
}

#[get("/{team_id}")]
async fn find_by_id(
    connections: ConnectionsHandle,
    team_id: web::Path<i32>,
) -> actix_web::Result<impl Responder> {
    let id = team_id.into_inner();
    let team = web::block(move || {
        let connections = connections.lock().unwrap();
        let mut connection = connections
            .db
            .get()
            .expect("Couldn't get db connection from pool");
        crate::model::operations::teams::find_by_id(&mut connection, id)
    })
    .await?
    .map_err(error::ErrorInternalServerError)?;
    match team {
        Some(team) => Ok(HttpResponse::Ok().json(team)),
        None => Ok(HttpResponse::NotFound().into()),
    }
}

#[delete("/{team_id}")]
async fn delete(
    connections: ConnectionsHandle,
    team_id: web::Path<i32>,
) -> actix_web::Result<impl Responder> {
    let id = team_id.into_inner();
    let deleted_team_count = web::block(move || {
        let connections = connections.lock().unwrap();
        let mut connection = connections
            .db
            .get()
            .expect("Couldn't get db connection from pool");
        crate::model::operations::teams::delete(&mut connection, id)
    })
    .await?
    .map_err(error::ErrorInternalServerError)?;
    match deleted_team_count {
        0 => Ok(HttpResponse::NotFound()),
        _ => Ok(HttpResponse::Ok()),
    }
}
