use serde::{Deserialize, Serialize};
use uuid::Uuid;

#[derive(Serialize, Deserialize)]
/// Helper structure that automatically fills the structure to be saved in database
/// The fields specific to the [`ProjectRequestTypeDTO`] are prefixed by `request_type_{request_type}`
pub struct NewProjectRequestDTO {
    /// A comment field to add additional information to the request
    pub comment: Option<String>,
    /// The type of the request
    pub project_request_type: ProjectRequestTypeDTO,
    /// The id of the targeted project, it is none only in the case of a request of type creation AND when the project hasn't been created yet
    pub project_id: Option<Uuid>,
    /// The name of the project, specific to the request type [`ProjectRequestTypeDTO::Creation`]
    pub request_type_creation_project_name: Option<String>,
}

#[derive(Serialize, Deserialize)]
pub enum ProjectRequestTypeDTO {
    /// Storage quota extension
    Quota,
    /// Storage duration deadline
    Duration,
    /// Project creation
    Creation,
    Reservation,
}
