use crate::views::{
    authentication_wrapper::AuthenticationWrapper,
    data_management_policy::AcceptDataManagementPolicy,
    forms::{NewProject, QuotaExtension, ValidityDateExtension},
    header::AuthHeader,
    home::Home,
    login::Login,
    not_found::NotFound,
};
use dioxus::prelude::*;
use dioxus_router::prelude::*;

#[derive(Routable, Clone)]
pub enum Route {
    #[layout(AuthHeader)]
    #[route("/")]
    Home {},

    // https://dioxuslabs.com/learn/0.4/router/reference/routes#query-segments
    #[route("/login?:code")]
    Login { code: String },
    #[layout(AuthenticationWrapper)]
    #[route("/new_project")]
    NewProject {},
    #[end_layout]
    #[route("/quota_extension")]
    QuotaExtension {},
    #[route("/validity_date_extension")]
    ValidityDateExtension {},
    #[route("/data_management_policy")]
    AcceptDataManagementPolicy {},
    #[end_layout]
    #[route("/:..route")]
    NotFound { route: Vec<String> },
}
