use dmp_dto::project_requests::{NewProjectRequestDTO, ProjectRequestTypeDTO};
use openidconnect::AccessToken;

use super::api_post_request;

/// Helper function to make a generic project request api call, calls the `\project_requests`
///
/// `access_token`: The openidconnect access token
///
/// `backend_url`: The base url of the backend
///
/// `new_project_request`: The new request we want to make
pub async fn add_new_project_request(
    access_token: AccessToken,
    backend_url: String,
    new_project_request: NewProjectRequestDTO,
) -> Result<reqwest::Response, reqwest::Error> {
    let endpoint = String::from("/project_requests");
    api_post_request(access_token, backend_url, endpoint, new_project_request).await
}

/// Calls the project_request api endpoint to request the creation of a new project, calls the endpoint specified in [`add_new_project_request`]
///
/// `access_token`: The openidconnect access token
///
/// `backend_url`: The base url of the backend
///
/// `project_name`: The name of the project
///
/// `project_comment`: Additional information on the request
pub async fn add_new_project_request_creation(
    access_token: AccessToken,
    backend_url: String,
    project_name: String,
    project_comment: Option<String>,
) -> Result<reqwest::Response, reqwest::Error> {
    let new_project_request = NewProjectRequestDTO {
        comment: project_comment,
        project_request_type: ProjectRequestTypeDTO::Creation,
        project_id: None,
        request_type_creation_project_name: Some(project_name),
    };
    add_new_project_request(access_token, backend_url, new_project_request).await
}
