use dmp_dto::UserWithTeamDTO;
use openidconnect::AccessToken;

/// Adds an user alongside their team, this function calls the `{backend_url}/users/add_user_and_team` endpoint.
///
/// # Arguments
///
/// *`access_token` - An access token retrieved using the authentication flow
///
/// *`backend_url` - The url of the backend hosting the database
///
/// *`uid_number` - The id of the user
///
/// *`gid_number` - The id of the group the user belongs to
pub async fn add_user_and_team(
    access_token: AccessToken,
    backend_url: String,
    user_id: i32,
    team_id: i32,
) -> Result<reqwest::Response, reqwest::Error> {
    let client = reqwest::Client::new();
    let path = "/users/add_user_and_team";
    let url = format!("{backend_url}{path}");
    let user_with_team = UserWithTeamDTO {
        posix_team_id: team_id,
        team_name: String::from("Dioxus Team name"),
        posix_user_id: user_id,
        username: String::from("Dioxus username"),
        policy_accepted: false,
    };
    client
        .post(url)
        .json(&user_with_team)
        .bearer_auth(access_token.secret())
        .send()
        .await
}
