use dioxus::prelude::*;
use dioxus_router::components::Link;
use reqwest::StatusCode;

use crate::router::Route;

#[derive(Props)]
pub struct FormResponseProps<'a> {
    result: &'a Result<reqwest::Response, reqwest::Error>,
}

/// A generic component used to display a generic message after a form request was made
#[component]
pub fn FormResponse<'a>(cx: Scope<'a, FormResponseProps>) -> Element<'a> {
    cx.render(rsx! {
        match cx.props.result {
            Ok(response) => {
                if response.status() == StatusCode::OK {
                    rsx! { div { "Form submitted successfully" } }
                } else {
                    rsx! { div { "An error was returned from the server: {response:?}" } }
                }
            }
            Err(error) => rsx! { div { "Something happened:{error:?}" } },
        },
        Link { to: Route::Home {}, "Go back home" }
    })
}
