use crate::{
    services::project_requests::add_new_project_request_creation,
    views::forms::form_response::FormResponse, DMP_BACK_URL,
};
use dioxus::prelude::*;
use openidconnect::AccessToken;

/// The form used to request the creation of a new project
#[component]
pub fn NewProject(cx: Scope) -> Element {
    // The access token is only accessible if the user is authenticated
    let access_token_state = use_shared_state::<AccessToken>(cx).unwrap();
    let access_token = access_token_state.read();
    let request_result = use_state(cx, || None);
    let new_project_creation = move |event: Event<FormData>| {
        cx.spawn({
            to_owned![access_token, request_result];
            async move {
                let values = event.values().clone();
                let project_name = values.get("project-name");
                let project_comment = values.get("project-comment");
                let comment = match project_comment {
                    Some(FormValue::Text(comment)) => Some(comment.clone()),
                    Some(FormValue::VecText(comment_block)) => Some(comment_block.join("")),
                    None => None,
                };
                let backend_url = DMP_BACK_URL.to_string();
                if let Some(FormValue::Text(name)) = project_name {
                    let response = add_new_project_request_creation(
                        access_token,
                        backend_url,
                        name.to_string(),
                        comment,
                    )
                    .await;
                    request_result.set(Some(response));
                }
            }
        })
    };

    cx.render(match request_result.get() {
        None => {
            rsx! {
                form { onsubmit: new_project_creation,
                    div {
                        label { r#for: "project-name", "Project name: " }
                        input { r#type: "text", name: "project-name", required: true }
                    }
                    div {
                        label { r#for: "project-comment", "Commentaire sur la demande: " }
                        textarea { name: "project-comment" }
                    }
                    input { r#type: "submit", value: "Submit", "Submit" }
                }
            }
        }
        Some(request_result) => rsx! { FormResponse { result: request_result } },
    })
}
