use crate::{
    oidc::{team_id, token_response, user_id, AuthRequestState, AuthTokenState},
    router::Route,
    services::user::add_user_and_team,
    storage::PersistentWrite,
    DIOXUS_FRONT_URL, DMP_BACK_URL, FERMI_AUTH_REQUEST, FERMI_AUTH_TOKEN, FERMI_CLIENT,
};
use dioxus::prelude::*;
use dioxus_router::prelude::{Link, NavigationTarget};
use fermi::*;
use openidconnect::{OAuth2TokenResponse, TokenResponse};

#[component]
pub fn Login(cx: Scope, code: String) -> Element {
    let fermi_client = use_atom_ref(cx, &FERMI_CLIENT);
    let fermi_auth_token = use_atom_ref(cx, &FERMI_AUTH_TOKEN);
    let home_url: NavigationTarget<Route> = DIOXUS_FRONT_URL.parse().unwrap();
    let fermi_auth_request = use_atom_ref(cx, &FERMI_AUTH_REQUEST);
    let fermi_auth_request_read = fermi_auth_request.read().clone();
    let client = fermi_client.read().oidc_client.clone();
    let auth_token_read = fermi_auth_token.read().clone();
    cx.render(match (client, auth_token_read) {
        (Some(client_props), Some(auth_token_read)) => {
            match (auth_token_read.id_token, auth_token_read.refresh_token, auth_token_read.access_token) {
                (Some(id_token), Some(_refresh_token), Some(access_token)) => {
                    let exchange_refresh_token_spawn = move || {
                        cx.spawn({
                            async move {
                                let dmp_client = client_props.client;
                                let nonce = fermi_auth_request_read.unwrap().auth_request.unwrap().nonce;
                                let user_id = user_id(dmp_client.clone(), id_token.clone(), nonce.clone()).unwrap();
                                let team_id = team_id(dmp_client, id_token, nonce).unwrap();

                                match add_user_and_team(access_token, DMP_BACK_URL.to_string(),user_id, team_id).await{
                                    Ok(response) => {
                                        log::info!("{response:?}");
                                    }
                                    Err(error) => {
                                        log::info!("{error:?}");
                                    }
                                }
                            }})};
                            exchange_refresh_token_spawn();
                    rsx! {
                        div { "Sign in successful" }
                        Link { to: home_url, "Go back home" }
                    }
                }
                (None, None, None) => {
                    let auth_code = code.to_string();
                    let token_response_spawn = move ||{
                        cx.spawn({
                            let fermi_auth_token = fermi_auth_token.to_owned();
                            async move {
                                let token_response_result = token_response(client_props.client, auth_code).await;
                                match token_response_result {
                                    Ok(token_response) => {
                                        let id_token = token_response.id_token().unwrap();
                                        AuthTokenState::persistent_set(&fermi_auth_token, Some(AuthTokenState {
                                            id_token: Some(id_token.clone()),
                                            refresh_token: token_response.refresh_token().cloned(),
                                            access_token: Some(token_response.access_token().clone())
                                        }));
                                    }
                                    Err(error) => {
                                        log::warn!{"Failed to decode token: {error}"};
                                    }
                                }
                            }
                        })
                    };
                    token_response_spawn();
                    rsx!{{}}
                }
                // If any of the token wasn't set, there was an error, we just go back home and reset their value
                _ => {
                    rsx! {
                        div { "Error while attempting to log in" }
                        Link {
                            to: home_url,
                            onclick: move |_| {
                                AuthTokenState::persistent_set(fermi_auth_token, Some(AuthTokenState::default()));
                                AuthRequestState::persistent_set(
                                    fermi_auth_request,
                                    Some(AuthRequestState::default()),
                                );
                            },
                            "Go back home"
                        }
                    }
                }
            }
        }
        (_, _) => {
            rsx! {{}}
        }
    })
}
